/* This module is responsible for the initial setup of the MAC registers and for sequencing the resets as needed.

Resets:

Initialization:
  Write MAC address to MAC registers
  Setup interrupts, if desired (not...)
  Setup the clock divider for MII
  Setup the descriptors
  Setup the mode register   

*/
module mac_control_init(sysclk,start,active,reset_phy,reset_wb,addr,data_out,
                              cycle,strobe,ack,MYMAC);
    input start;
    input sysclk;
	 input [47:0] MYMAC;
    output reg active;
    output [11:2] addr;
    output reg [31:0] data_out;
    output cycle, strobe;
    input ack;
    output reg reset_wb;
    output reg reset_phy;

	reg [11:0] addrReg;
	reg done;
	reg [7:0] operation_steps;
	reg [1:0] ministeps;
	reg [21:0] delay_counter;

assign addr[11:2]=addrReg[11:2];

always @(posedge sysclk) reset_phy<=(operation_steps==4'h1);
always @(posedge sysclk) reset_wb<=(operation_steps==4'h2);
always @(posedge sysclk) active<=!done;

always @(posedge sysclk) begin
	case (operation_steps)
		8'h4 : begin data_out[31:16]<=16'h0; data_out[15:8]<=MYMAC[7:0]; data_out[7:0]<=MYMAC[15:8]; addrReg<=12'h44; end// MAC 1
		8'h5 : begin data_out[31:24]<=MYMAC[23:16]; data_out[23:16]<=MYMAC[31:24]; data_out[15:8]<=MYMAC[39:32]; data_out[7:0]<=MYMAC[47:40]; addrReg<=12'h40; end// MAC 0
		8'h6 : begin data_out[31:0]<=32'h0; addrReg<=12'h04; end
		8'h7 : begin data_out[31:0]<=32'h0; addrReg<=12'h08; end
		// : begin data_out[31:0]=32'h0; addrReg<=12'h28; end; //clk divider,etc
		8'hb : begin  // RX
		 	data_out[31:16]<=16'h0; 
		 	data_out[15]<=1'h0;  // not ready
			data_out[14]<=1'h0;  // no interrupt
			data_out[13]<=1'h1;  // last in table
			data_out[12:9]<=0;  // blah, blah
			data_out[8:0]<=9'h0; // clear status history
			addrReg<=12'h600;
		end
		8'hc : begin // RX base reg
			addrReg<=12'h604;
			data_out[31:0]<=32'h1f800;  // 63...
		end

// not last in table data_out[31:0] = 32'h00001800
// last in table data_out[31:0] = 32'h00003800

		8'h10 : begin  // TX 0
			data_out[31:0]<=32'h00001800;
			addrReg<=12'h400;
		end
		8'h11 : addrReg<=12'h408; // TX 1 
		8'h12 : addrReg<=12'h410; // TX 2 
		8'h13 : addrReg<=12'h418; // TX 3 
		8'h14 : addrReg<=12'h420; // TX 4 
		8'h15 : addrReg<=12'h428; // TX 5 
		8'h16 : addrReg<=12'h430; // TX 6 
		8'h17 : begin 
			data_out[31:0]<=32'h00003800; // last!
			addrReg<=12'h438; // TX 7 
	   end

		8'h80 : begin 	addrReg<=12'h404;  data_out[31:0]<=32'h000000;  end	 // TX 0
		8'h81 : begin 	addrReg<=12'h40C;  data_out[31:0]<=32'h000800;  end	 // TX 1
		8'h82 : begin 	addrReg<=12'h414;  data_out[31:0]<=32'h001000;  end	 // TX 2
		8'h83 : begin 	addrReg<=12'h41C;  data_out[31:0]<=32'h001800;  end	 // TX 3
		8'h84 : begin 	addrReg<=12'h424;  data_out[31:0]<=32'h002000;  end	 // TX 4
		8'h85 : begin 	addrReg<=12'h42C;  data_out[31:0]<=32'h002800;  end	 // TX 5
		8'h86 : begin 	addrReg<=12'h434;  data_out[31:0]<=32'h003000;  end	 // TX 6
		8'h87 : begin 	addrReg<=12'h43C;  data_out[31:0]<=32'h003800;  end	 // TX 7

		8'hFE : begin // MODER
			data_out[31:17]<=0; // reserved
			data_out[16]<=1'h0; // ignore short packets
			data_out[15]<=1'h1; // pad short frames
			data_out[14]<=1'h0; // jumbo disable
			data_out[13]<=1'h1; // CRC enable
			data_out[12]<=1'h0; // "normal" CRC
			data_out[11]<=1'h0; // reserved
			data_out[10]<=1'h1; // full duplex
			data_out[9]<=1'h1; // wait for carrier
			data_out[8]<=1'h0; // normal backoff
			data_out[7]<=1'h0; // no loopback
			data_out[6]<=1'h0; // normal IFG
			data_out[5]<=1'h0; // non-promisc
			data_out[4]<=1'h0; // normal IAM checking (I'm a host)
			data_out[3]<=1'h1; // reject broadcast
			data_out[2]<=1'h0; // normal preamble
			data_out[1]<=1'h1; // TX enable
			data_out[0]<=1'h1; // RX enable
			addrReg<=12'h0;
		end
	endcase 
end

always @(posedge sysclk)
	if (start) done<=0;
	else if (operation_steps==8'hFF) done<=1;

always @(posedge sysclk) 
	if (operation_steps<4 || done) ministeps<=0;
	else if (ministeps==0) ministeps<=1;
	else if (ministeps==1) ministeps<=2;
	else if (ministeps==2 && ack) ministeps<=3;
	else if (ministeps==3) ministeps<=0;

assign cycle=(operation_steps!=0 && !done);
assign strobe=(ministeps==1 || ministeps==2);

always @(posedge sysclk)
	if (delay_counter==0) begin
		if (operation_steps==0) delay_counter<=0;
		else	if (operation_steps==1) delay_counter<=22'h300;
		else if (operation_steps==2) delay_counter<=22'h300;
		else if (operation_steps==3) delay_counter<=22'h4; // short delay
		else delay_counter<=0;
	end else delay_counter<=delay_counter-1;

always @(posedge sysclk) 
	if (start) operation_steps<=1;
	else if (done) operation_steps<=8'hFF;
	else if (operation_steps>=4 && ministeps==0) operation_steps<=operation_steps+1;
	else if (operation_steps<4 && delay_counter==1) operation_steps<=operation_steps+1;

endmodule
